@@ -27,7 +27,6 @@ from oidcmsg.exception import NotForMe
 from oidcmsg.exception import OidcMsgError
 from oidcmsg.exception import SchemeError
 from oidcmsg.exception import VerificationError
-from oidcmsg.message import Message
 from oidcmsg.message import OPTIONAL_LIST_OF_SP_SEP_STRINGS
 from oidcmsg.message import OPTIONAL_LIST_OF_STRINGS
 from oidcmsg.message import OPTIONAL_MESSAGE
@@ -37,6 +36,7 @@ from oidcmsg.message import SINGLE_OPTIONAL_INT
 from oidcmsg.message import SINGLE_OPTIONAL_JSON
 from oidcmsg.message import SINGLE_OPTIONAL_STRING
 from oidcmsg.message import SINGLE_REQUIRED_STRING
+from oidcmsg.message import Message
 from oidcmsg.message import msg_ser
 from oidcmsg.oauth2 import ResponseMessage
 from oidcmsg.time_util import utc_time_sans_frac
@@ -299,7 +299,8 @@ def verify_id_token(msg, check_hash=False, claim='id_token', **kwargs):
     if _signed and 'keyjar' in kwargs:
         try:
             if _body['iss'] not in kwargs['keyjar']:
-                raise ValueError('Unknown issuer')
+                logger.info('KeyJar issuers: {}'.format(kwargs['keyjar']))
+                raise ValueError('Unknown issuer: "{}"'.format(_body['iss']))
         except KeyError:
             raise MissingRequiredAttribute('iss')
 
@@ -625,7 +626,9 @@ class RegistrationRequest(Message):
         "frontchannel_logout_uri": SINGLE_OPTIONAL_STRING,
         "frontchannel_logout_session_required": SINGLE_OPTIONAL_BOOLEAN,
         "backchannel_logout_uri": SINGLE_OPTIONAL_STRING,
-        "backchannel_logout_session_supported": SINGLE_OPTIONAL_BOOLEAN
+        "backchannel_logout_session_supported": SINGLE_OPTIONAL_BOOLEAN,
+        "federation_type": OPTIONAL_LIST_OF_STRINGS,
+        "organization_name": SINGLE_OPTIONAL_STRING
     }
     c_default = {"application_type": "web", "response_types": ["code"]}
     c_allowed_values = {
@@ -821,6 +824,11 @@ class IdToken(OpenIDSchema):
         else:
             if (_iat + _storage_time) < (_now - _skew):
                 raise IATError('Issued too long ago')
+            elif _iat > _now + _skew:
+                raise IATError('Issued sometime in the future')
+
+        if _exp < _iat:
+            raise IATError('Expiration time can not be earlier the issued at')
 
         if 'nonce' in kwargs and 'nonce' in self:
             if kwargs['nonce'] != self['nonce']:
@@ -922,6 +930,14 @@ class ProviderConfigurationResponse(ResponseMessage):
         elif parts.scheme != "https":
             raise SchemeError("Not HTTPS")
 
+        # The parameter is optional
+        if "token_endpoint_auth_signing_alg_values_supported" in self and "none" in self[
+                "token_endpoint_auth_signing_alg_values_supported"]:
+            raise ValueError(
+                "The value none must not be used for "
+                "token_endpoint_auth_signing_alg_values_supported"
+            )
+
         if "RS256" not in self["id_token_signing_alg_values_supported"]:
             raise ValueError('RS256 missing from id_token_signing_alg_values_supported')
 
@@ -931,7 +947,7 @@ class ProviderConfigurationResponse(ResponseMessage):
             raise ValueError('Issuer ID invalid')
 
         if any("code" in rt for rt in self[
-            "response_types_supported"]) and "token_endpoint" not in self:
+                "response_types_supported"]) and "token_endpoint" not in self:
             raise MissingRequiredAttribute("token_endpoint")
 
         return True
@@ -1154,7 +1170,8 @@ def factory(msgtype, **kwargs):
     return oauth2.factory(msgtype, **kwargs)
 
 
-def make_openid_request(arq, keys, issuer, request_object_signing_alg, recv):
+def make_openid_request(arq, keys, issuer, request_object_signing_alg, recv, with_jti=False,
+                        lifetime=0):
     """
     Construct the JWT to be passed by value (the request parameter) or by
     reference (request_uri).
@@ -1165,10 +1182,16 @@ def make_openid_request(arq, keys, issuer, request_object_signing_alg, recv):
     :param issuer: Who is signing this JSON Web Token
     :param request_object_signing_alg: Which signing algorithm to use
     :param recv: The intended receiver of the request
+    :param with_jti: Whether a JTI should be included in the JWT.
+    :param lifetime: How long the JWT is expect to be live.
     :return: JWT encoded OpenID request
     """
 
     _jwt = JWT(key_jar=keys, iss=issuer, sign_alg=request_object_signing_alg)
+    if with_jti:
+        _jwt.with_jti = True
+    if lifetime:
+        _jwt.lifetime = lifetime
     return _jwt.pack(arq.to_dict(), owner=issuer, recv=recv)
 
 
